@@ -19,10 +19,10 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = '0.0.43'
+__version__ = '0.0.44'
 
 # Used with pypi checks and other messages related to pip
-_wandb_module = 'wandb-ng'
+_wandb_module = 'wandb'
 
 import sys
 
@@ -50,7 +50,7 @@ Settings = wandb_sdk.Settings
 Config = wandb_sdk.Config
 
 from wandb.apis import InternalApi, PublicApi
-from wandb.errors.error import CommError
+from wandb.errors.error import CommError, UsageError
 
 from wandb.lib import preinit as _preinit
 from wandb.lib import lazyloader as _lazyloader
@@ -73,6 +73,7 @@ from wandb.wandb_agent import agent
 from wandb.wandb_controller import sweep, controller
 
 from wandb import superagent
+from wandb.jupyteragent import jupyteragent as _secretagent
 
 # from wandb.core import *
 from wandb.viz import visualize
@@ -109,22 +110,25 @@ run = None
 config = _preinit.PreInitObject("wandb.config")
 summary = _preinit.PreInitObject("wandb.summary")
 log = _preinit.PreInitCallable(
-    "wandb.log", wandb_sdk.wandb_run.RunManaged.log
+    "wandb.log", wandb_sdk.wandb_run.Run.log
 )
 join = _preinit.PreInitCallable(
-    "wandb.join", wandb_sdk.wandb_run.RunManaged.join
+    "wandb.join", wandb_sdk.wandb_run.Run.join
+)
+finish = _preinit.PreInitCallable(
+    "wandb.finish", wandb_sdk.wandb_run.Run.finish
 )
 save = _preinit.PreInitCallable(
-    "wandb.save", wandb_sdk.wandb_run.RunManaged.save
+    "wandb.save", wandb_sdk.wandb_run.Run.save
 )
 restore = _preinit.PreInitCallable(
-    "wandb.restore", wandb_sdk.wandb_run.RunManaged.restore
+    "wandb.restore", wandb_sdk.wandb_run.Run.restore
 )
 use_artifact = _preinit.PreInitCallable(
-    "wandb.use_artifact", wandb_sdk.wandb_run.RunManaged.use_artifact
+    "wandb.use_artifact", wandb_sdk.wandb_run.Run.use_artifact
 )
 log_artifact = _preinit.PreInitCallable(
-    "wandb.log_artifact", wandb_sdk.wandb_run.RunManaged.log_artifact
+    "wandb.log_artifact", wandb_sdk.wandb_run.Run.log_artifact
 )
 # record of patched libraries
 patched = {"tensorboard": [], "keras": [], "gym": []}
