@@ -3,6 +3,9 @@
 api.
 """
 
+import sys
+from wandb import util
+util.vendor_setup()
 from .internal import Api as InternalApi
 from .public import Api as PublicApi
 
